@@ -1,5 +1,5 @@
 /*
- * Copyright © 2020-2022 Inria.  All rights reserved.
+ * Copyright © 2020-2023 Inria.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
@@ -14,6 +14,53 @@
 #include <level_zero/ze_api.h>
 #include <level_zero/zes_api.h>
 
+struct hwloc_osdev_array {
+  unsigned nr, nr_allocated;
+  hwloc_obj_t *objs;
+};
+
+static void
+hwloc__levelzero_osdev_array_init(struct hwloc_osdev_array *array)
+{
+  array->nr = 0;
+  array->nr_allocated = 0;
+  array->objs = NULL;
+}
+
+static int
+hwloc__levelzero_osdev_array_add(struct hwloc_osdev_array *array,
+                                 hwloc_obj_t new)
+{
+  if (array->nr_allocated == array->nr) {
+    unsigned new_nr_allocated = array->nr_allocated + 40; /* enough for 8 devices + 4 subdevices each on first allocation */
+    hwloc_obj_t *tmp = realloc(array->objs, new_nr_allocated * sizeof(*array->objs));
+    if (!tmp)
+      /* the object won't be added */
+      return -1;
+    array->nr_allocated = new_nr_allocated;
+    array->objs = tmp;
+  }
+  array->objs[array->nr++] = new;
+  return 0;
+}
+
+static void
+hwloc__levelzero_osdev_array_free(struct hwloc_osdev_array *array)
+{
+  free(array->objs);
+}
+
+static int
+hwloc__levelzero_osdev_array_find(struct hwloc_osdev_array *array,
+                                  hwloc_obj_t osdev)
+{
+  unsigned i;
+  for(i=0; i<array->nr; i++)
+    if (array->objs[i] == osdev)
+      return (int)i;
+  return -1;
+}
+
 static void
 hwloc__levelzero_properties_get(ze_device_handle_t h, hwloc_obj_t osdev,
                                 int sysman_maybe_missing,
@@ -43,7 +90,7 @@ hwloc__levelzero_properties_get(ze_device_handle_t h, hwloc_obj_t osdev,
     case ZE_DEVICE_TYPE_MCA: type = "MCA"; break;
     case ZE_DEVICE_TYPE_VPU: type = "VPU"; break;
     default:
-      if (!hwloc_hide_errors())
+      if (HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/levelzero: unexpected device type %u\n", (unsigned) prop.type);
       type = "Unknown";
     }
@@ -93,9 +140,9 @@ hwloc__levelzero_properties_get(ze_device_handle_t h, hwloc_obj_t osdev,
   } else {
     static int warned = 0;
     if (!warned) {
-      if (sysman_maybe_missing == 1 && !hwloc_hide_errors())
+      if (sysman_maybe_missing == 1 && HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/levelzero: zesDeviceGetProperties() failed (ZES_ENABLE_SYSMAN=1 set too late?).\n");
-      else if (sysman_maybe_missing == 2 && !hwloc_hide_errors())
+      else if (sysman_maybe_missing == 2 && HWLOC_SHOW_ALL_ERRORS())
         fprintf(stderr, "hwloc/levelzero: zesDeviceGetProperties() failed (ZES_ENABLE_SYSMAN=0).\n");
       warned = 1;
     }
@@ -180,8 +227,8 @@ hwloc__levelzero_memory_get_from_sysman(zes_device_handle_t h,
 
           if (mprop.onSubdevice) {
             if (mprop.subdeviceId >= nr_osdevs || !nr_osdevs || !sub_osdevs) {
-              if (!hwloc_hide_errors())
-                fprintf(stderr, "LevelZero: memory module #%u on unexpected subdeviceId #%u\n", m, mprop.subdeviceId);
+              if (HWLOC_SHOW_ALL_ERRORS())
+                fprintf(stderr, "hwloc/levelzero: memory module #%u on unexpected subdeviceId #%u\n", m, mprop.subdeviceId);
               osdev = NULL; /* we'll ignore it but we'll still agregate its subdevice memories into totalHBM/DDRkB */
             } else {
               osdev = sub_osdevs[mprop.subdeviceId];
@@ -330,6 +377,217 @@ hwloc__levelzero_memory_get(zes_device_handle_t h, hwloc_obj_t root_osdev, int i
   }
 }
 
+struct hwloc_levelzero_ports {
+  unsigned nr_allocated;
+  unsigned nr;
+  struct hwloc_levelzero_port {
+    hwloc_obj_t osdev;
+    zes_fabric_port_properties_t props;
+    zes_fabric_port_state_t state;
+  } *ports;
+};
+
+static void
+hwloc__levelzero_ports_init(struct hwloc_levelzero_ports *hports)
+{
+  hports->nr_allocated = 0;
+  hports->nr = 0;
+  hports->ports = NULL;
+
+  free(hports->ports);
+}
+
+static void
+hwloc__levelzero_ports_get(zes_device_handle_t dvh,
+                           hwloc_obj_t root_osdev,
+                           unsigned nr_sub_osdevs, hwloc_obj_t *sub_osdevs,
+                           struct hwloc_levelzero_ports *hports)
+{
+  zes_fabric_port_handle_t *ports;
+  uint32_t nr_new = 0;
+  unsigned i;
+  ze_result_t res;
+
+  res = zesDeviceEnumFabricPorts(dvh, &nr_new, NULL);
+  if (res != ZE_RESULT_SUCCESS || !nr_new)
+    return;
+  hwloc_debug("L0 device %s has %u fabric ports\n", root_osdev->name, nr_new);
+
+  if (hports->nr_allocated - hports->nr < nr_new) {
+    /* we must extend the array */
+    struct hwloc_levelzero_port *tmp;
+    uint32_t new_nr_allocated;
+    /* Extend the array by 8x the number of ports in this device.
+     * This should mean a single allocation per 8 devices.
+     */
+    new_nr_allocated = hports->nr_allocated + 8*nr_new;
+    tmp = realloc(hports->ports, new_nr_allocated * sizeof(*hports->ports));
+    if (!tmp)
+      return;
+    hports->ports = tmp;
+    hports->nr_allocated = new_nr_allocated;
+  }
+
+  ports = malloc(nr_new * sizeof(*ports));
+  if (!ports)
+    return;
+  res = zesDeviceEnumFabricPorts(dvh, &nr_new, ports);
+
+  for(i=0; i<nr_new; i++) {
+    unsigned id = hports->nr;
+    res = zesFabricPortGetProperties(ports[i], &hports->ports[id].props);
+    if (res != ZE_RESULT_SUCCESS)
+      continue;
+    if (hports->ports[id].props.onSubdevice) {
+      hwloc_debug("  port #%u is on subdevice #%u\n",
+                  i, hports->ports[id].props.subdeviceId);
+      if (hports->ports[id].props.subdeviceId < nr_sub_osdevs) {
+        hports->ports[id].osdev = sub_osdevs[hports->ports[id].props.subdeviceId];
+      } else {
+        hwloc_debug("    no such subdevice exists, ignoring\n");
+        continue;
+      }
+      hports->ports[id].osdev = root_osdev;
+    }
+
+    res = zesFabricPortGetState(ports[i], &hports->ports[id].state);
+    if (res != ZE_RESULT_SUCCESS)
+      continue;
+    if (hports->ports[id].state.status != ZES_FABRIC_PORT_STATUS_HEALTHY
+        && hports->ports[id].state.status != ZES_FABRIC_PORT_STATUS_DEGRADED) {
+      hwloc_debug("    status (%u)  isn't healthy or degraded\n",
+                  (unsigned)hports->ports[id].state.status);
+      continue;
+    }
+    hwloc_debug("    status is %s, this will be our hport #%u\n",
+                hports->ports[id].state.status == ZES_FABRIC_PORT_STATUS_HEALTHY ? "healthy" : "degraded",
+                id);
+
+    hwloc_debug("    portId %u-%u-%u is connected to remote portId %u-%u-%u\n",
+                hports->ports[id].props.portId.fabricId,
+                hports->ports[id].props.portId.attachId,
+                hports->ports[id].props.portId.portNumber,
+                hports->ports[id].state.remotePortId.fabricId,
+                hports->ports[id].state.remotePortId.attachId,
+                hports->ports[id].state.remotePortId.portNumber);
+    hports->nr++;
+  }
+
+  free(ports);
+}
+
+static int
+hwloc__levelzero_ports_add_xelink_bandwidth(struct hwloc_topology *topology,
+                                            struct hwloc_osdev_array *oarray,
+                                            hwloc_uint64_t *bws)
+{
+  void *handle;
+  int err;
+
+  handle = hwloc_backend_distances_add_create(topology, "XeLinkBandwidth",
+                                              HWLOC_DISTANCES_KIND_FROM_OS|HWLOC_DISTANCES_KIND_MEANS_BANDWIDTH,
+                                              0);
+  if (!handle)
+    goto out;
+
+  err = hwloc_backend_distances_add_values(topology, handle, oarray->nr, oarray->objs, bws, 0);
+  if (err < 0)
+    goto out;
+  /* arrays are now attached to the handle */
+  oarray->objs = NULL;
+  bws = NULL;
+
+  err = hwloc_backend_distances_add_commit(topology, handle, 0 /* don't group GPUs */);
+  if (err < 0)
+    goto out;
+
+  return 0;
+
+ out:
+  free(bws);
+  return -1;
+}
+
+static int
+hwloc__levelzero_ports_connect(struct hwloc_topology *topology,
+                               struct hwloc_osdev_array *oarray,
+                               struct hwloc_levelzero_ports *hports)
+{
+  hwloc_uint64_t *bws;
+  unsigned i, j, k;
+  int gotbw = 0;
+
+  if (!hports->nr)
+    return 0;
+
+  bws = calloc(oarray->nr * oarray->nr, sizeof(*bws));
+  if (!bws)
+    return -1;
+
+  for(i=0; i<hports->nr; i++) {
+    if (hports->ports[i].state.status == ZES_FABRIC_PORT_STATUS_HEALTHY) {
+      for(j=0; j<hports->nr; j++) {
+        if (i==j)
+          continue;
+        if (hports->ports[i].state.remotePortId.fabricId == hports->ports[j].props.portId.fabricId
+            && hports->ports[i].state.remotePortId.attachId == hports->ports[j].props.portId.attachId
+            && hports->ports[i].state.remotePortId.portNumber == hports->ports[j].props.portId.portNumber) {
+          int iindex, jindex;
+          hwloc_debug("found L0 link model %s with %llu bit/s TX from hport #%u (osdev %s) to hport #%u (osdev %s)\n",
+                      hports->ports[j].props.model,
+                      (unsigned long long) hports->ports[i].state.rxSpeed.bitRate,
+                      i,
+                      hports->ports[i].osdev->name,
+                      j,
+                      hports->ports[j].osdev->name);
+          /* only keep XeLink for now */
+          if (strcmp(hports->ports[j].props.model, "XeLink"))
+            continue;
+          iindex = hwloc__levelzero_osdev_array_find(oarray, hports->ports[i].osdev);
+          jindex = hwloc__levelzero_osdev_array_find(oarray, hports->ports[j].osdev);
+          if (iindex<0 || jindex<0)
+            continue;
+          bws[iindex*oarray->nr+jindex] += hports->ports[i].state.rxSpeed.bitRate >> 20; /* MB/s */
+          /* TODO: way to accumulate subdevs bw into rootdevs? tranformation? different matrix? */
+          gotbw++;
+        }
+      }
+    }
+  }
+  if (!gotbw) {
+    free(bws);
+    return 0;
+  }
+
+  /* Add very high artifical values on the diagonal since local is faster than remote.
+   * Use 1TB/s for local, it somehow matches the HBM.
+   * Also between subdevices of the same parent, and between subdevice and parent (TODO improve this)
+   *
+   * WARNING: this assumes that parent and subdevices were added together
+   * in oarray[] in hwloc_levelzero_discover().
+   */
+  for(i=0; i<oarray->nr; i++) {
+    unsigned nrsd = 0;
+    const char *s = hwloc_obj_get_info_by_name(oarray->objs[i], "LevelZeroSubdevices");
+    if (s)
+      nrsd = atoi(s);
+    /* set 1000000 for all devices between i and i+nrsd */
+    for(j=0; j<nrsd+1; j++)
+      for(k=0; k<nrsd+1; k++)
+        bws[(i+j)*oarray->nr+(i+k)] = 1000000;
+    /* skip subdevices */
+    i += nrsd;
+  }
+
+  return hwloc__levelzero_ports_add_xelink_bandwidth(topology, oarray, bws);
+}
+
+static void
+hwloc__levelzero_ports_destroy(struct hwloc_levelzero_ports *hports)
+{
+  free(hports->ports);
+}
+
 static int
 hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status *dstatus)
 {
@@ -344,9 +602,15 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
   ze_result_t res;
   ze_driver_handle_t *drh;
   uint32_t nbdrivers, i, k, zeidx;
+  struct hwloc_osdev_array oarray;
+  struct hwloc_levelzero_ports hports;
   int sysman_maybe_missing = 0; /* 1 if ZES_ENABLE_SYSMAN=1 was NOT set early, 2 if ZES_ENABLE_SYSMAN=0 */
   char *env;
 
+  hwloc__levelzero_osdev_array_init(&oarray);
+
+  hwloc__levelzero_ports_init(&hports);
+
   assert(dstatus->phase == HWLOC_DISC_PHASE_IO);
 
   hwloc_topology_get_type_filter(topology, HWLOC_OBJ_OS_DEVICE, &filter);
@@ -375,8 +639,8 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
 
   res = zeInit(0);
   if (res != ZE_RESULT_SUCCESS) {
-    if (!hwloc_hide_errors()) {
-      fprintf(stderr, "Failed to initialize LevelZero in ze_init(): 0x%x\n", (unsigned)res);
+    if (HWLOC_SHOW_ALL_ERRORS()) {
+      fprintf(stderr, "hwloc/levelzero: Failed to initialize in zeInit(): 0x%x\n", (unsigned)res);
     }
     return 0;
   }
@@ -452,8 +716,8 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
           zeDeviceGetSubDevices(dvh[j], &nr_subdevices, subh);
           for(k=0; k<nr_subdevices; k++) {
             subosdevs[k] = hwloc_alloc_setup_object(topology, HWLOC_OBJ_OS_DEVICE, HWLOC_UNKNOWN_INDEX);
-            snprintf(buffer, sizeof(buffer), "ze%u.%u", zeidx, k);
-            subosdevs[k]->name = strdup(buffer);
+            snprintf(tmp, sizeof(tmp), "ze%u.%u", zeidx, k);
+            subosdevs[k]->name = strdup(tmp);
             subosdevs[k]->depth = HWLOC_TYPE_DEPTH_UNKNOWN;
             subosdevs[k]->attr->osdev.type = HWLOC_OBJ_OSDEV_COPROC;
             subosdevs[k]->subtype = strdup("LevelZero");
@@ -461,7 +725,7 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
             snprintf(tmp, sizeof(tmp), "%u", k);
             hwloc_obj_add_info(subosdevs[k], "LevelZeroSubdeviceID", tmp);
 
-            hwloc__levelzero_properties_get(subh[j], subosdevs[k], sysman_maybe_missing, NULL);
+            hwloc__levelzero_properties_get(subh[k], subosdevs[k], sysman_maybe_missing, NULL);
 
             hwloc__levelzero_cqprops_get(subh[k], subosdevs[k]);
           }
@@ -476,6 +740,10 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
       /* get all memory info at once */
       hwloc__levelzero_memory_get(dvh[j], osdev, is_integrated, nr_subdevices, subh, subosdevs);
 
+      /* get all ports info at once */
+      if (!(hwloc_topology_get_flags(topology) & HWLOC_TOPOLOGY_FLAG_NO_DISTANCES))
+        hwloc__levelzero_ports_get(dvh[j], osdev, nr_subdevices, subosdevs, &hports);
+
       parent = NULL;
 #ifdef HWLOC_HAVE_ZEDEVICEPCIGETPROPERTIESEXT
       { /* try getting PCI BDF+speed from core extension */
@@ -515,11 +783,17 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
       if (!parent)
         parent = hwloc_get_root_obj(topology);
 
+      /* WARNING: parent and its subdevices are inserted together in oarray[].
+       * this is required when filling the BW matrix at the end of hwloc__levelzero_ports_connect()
+       */
       hwloc_insert_object_by_parent(topology, parent, osdev);
+      hwloc__levelzero_osdev_array_add(&oarray, osdev);
       if (nr_subdevices) {
         for(k=0; k<nr_subdevices; k++)
-          if (subosdevs[k])
+          if (subosdevs[k]) {
             hwloc_insert_object_by_parent(topology, osdev, subosdevs[k]);
+            hwloc__levelzero_osdev_array_add(&oarray, subosdevs[k]);
+          }
         free(subosdevs);
         free(subh);
       }
@@ -529,6 +803,11 @@ hwloc_levelzero_discover(struct hwloc_backend *backend, struct hwloc_disc_status
     free(dvh);
   }
 
+  hwloc__levelzero_ports_connect(topology, &oarray, &hports);
+  hwloc__levelzero_ports_destroy(&hports);
+
+  hwloc__levelzero_osdev_array_free(&oarray);
+
   free(drh);
   return 0;
 }
